﻿/* ============================================================
* Author:       liubingsheng
* Time:         2016/11/29 14:52:25
* FileName:     RedisSystem
* Purpose:      Redis
* =============================================================*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Utility
{
    /// <summary>
    /// Dictionary扩展
    /// </summary>
    public static class DictionaryExtension
    {
        /// <summary>
        /// 尝试添加元素
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="self"></param>
        /// <param name="key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public static bool TryAdd<TKey, TValue>(this Dictionary<TKey, TValue> self, TKey key, TValue value)
        {
            if (self.ContainsKey(key))
            {
                return false;
            }

            self.Add(key, value);

            return true;
        }

        /// <summary>
        /// 获得元素
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="self"></param>
        /// <param name="key"></param>
        /// <returns>返回类型的默认值，引用类型返回null</returns>
        public static TValue GetValue<TKey, TValue>(this Dictionary<TKey, TValue> self, TKey key)
        {
            if (!self.ContainsKey(key))
            {
                return default(TValue);

            }

            return self[key];
        }

        /// <summary>
        /// 尝试删除元素
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="self"></param>
        /// <param name="key"></param>
        /// <param name="value"></param>
        /// <returns></returns>
        public static bool TryRemove<TKey, TValue>(this Dictionary<TKey, TValue> self, TKey key)
        {
            if (!self.ContainsKey(key))
            {
                return false;
            }

            self.Remove(key);

            return true;
        }

        /// <summary>
        /// 获取目标字典通过键
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="selfDic"></param>
        /// <param name="getResult"></param>
        /// <returns></returns>
        public static Dictionary<TKey, TValue> GetResultDicByKey<TKey, TValue>(this Dictionary<TKey, TValue> selfDic, Func<TKey, bool> getResult)
        {
            Dictionary<TKey, TValue> resultDic = new Dictionary<TKey, TValue>();
            foreach (var item in selfDic)
            {
                if (getResult(item.Key))
                {
                    resultDic.Add(item.Key, item.Value);
                }
            }
            return resultDic;
        }

        /// <summary>
        /// 获取目标字典通过值
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="selfDic"></param>
        /// <param name="getResult"></param>
        /// <returns></returns>
        public static Dictionary<TKey, TValue> GetResultDicByValue<TKey, TValue>(this Dictionary<TKey, TValue> selfDic, Func<TValue, bool> getResult)
        {
            Dictionary<TKey, TValue> resultDic = new Dictionary<TKey, TValue>();
            foreach (var item in selfDic)
            {
                if (getResult(item.Value))
                {
                    resultDic.Add(item.Key, item.Value);
                }
            }
            return resultDic;
        }

        /// <summary>
        /// 删除所有
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="self"></param>
        /// <param name="keys"></param>
        /// <returns></returns>
        public static Dictionary<TKey, TValue> RemoveAll<TKey, TValue>(this Dictionary<TKey, TValue> self, IEnumerable<TKey> keys)
        {
            foreach(var key in keys)
            {
                self.Remove(key);
            }
            return self;
        }

        public static Dictionary<TKey, TValue> RemoveAll<TKey, TValue>(this Dictionary<TKey, TValue> self, Predicate<KeyValuePair<TKey, TValue>> match)
        {
            var lstKeys = new List<TKey>();
            foreach (var item in self)
            {
                if(match(item))
                {
                    lstKeys.Add(item.Key);
                }
            }
            self.RemoveAll(lstKeys);
            return self;
        }

        /// <summary>
        /// 是否包含
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="self"></param>
        /// <param name="match"></param>
        /// <returns></returns>
        public static bool Contains<TKey, TValue>(this Dictionary<TKey, TValue> self, Predicate<KeyValuePair<TKey, TValue>> match)
        {
            foreach (var item in self)
            {
                if (match(item))
                {
                    return true;
                }
            }

            return false;
        }

        /// <summary>
        /// 获取目标值通过权重字典
        /// </summary>
        /// <typeparam name="TKey"></typeparam>
        /// <typeparam name="TValue"></typeparam>
        /// <param name="selfDic"></param>
        /// <param name="getResult"></param>
        /// <returns></returns>
        public static TKey GetRandomValue<TKey>(this Dictionary<TKey, int> selfDic)
        {
            int totalCount = 0;
            foreach (var item in selfDic.Values)
            {
                totalCount += item;
            }
            int randomNum = RandomHelp.RandomInt(totalCount);
            foreach (var item in selfDic)
            {
                randomNum -= item.Value;
                if (randomNum < 0)
                {
                    return item.Key;
                }
            }
            return default(TKey);
        }
    }
}
